#include <ITDB02_Touch.h>
#include <ITDB02_Graph16.h>

#include "C5_Visual.h"
#include "C5_Button.h"

C5Button::C5Button()
{
  Reset();
}

void C5Button::Reset()
{
  C5Visual::Reset();
  _ticksTillAction = -1;
  RedrawOnPress = true;
}

void C5Button::Render(ITDB02 *pLCD)
{
}

void C5Button::Render(ITDB02 *pLCD, C5Position2 offset)
{
  C5Color bColor;
  C5Color color;
  
  if(IsPressed())
  {
    bColor = BackgroundSelected;
    color = ColorSelected;
  }
  else
  {
    bColor = Background;
    color = Color;
  }
  
  pLCD->setColor(bColor.R, bColor.G, bColor.B);
  pLCD->fillRoundRect(Position.X, Position.Y, Position.X + Size.Width, Position.Y + Size.Height);
  
  if(TextLength > 0)
  {
    pLCD->setFont(pFont);
    pLCD->setColor(color.R, color.G, color.B);
    pLCD->setBackColor(bColor.R, bColor.G, bColor.B);
    
    int textWidth = pLCD->cfont.x_size * TextLength;
    int x = (Size.Width - textWidth) >> 1;
    int y = (Size.Height - pLCD->cfont.y_size) >> 1;
    //char temp[100];
    //Text.toCharArray(temp, 100);
    
    pLCD->print(Text, x + Position.X + offset.X, y + Position.Y + offset.Y);
  }
}

bool C5Button::Update()
{
  if(_ticksTillAction >= 0)
  {
    _ticksTillAction--;
    if(_ticksTillAction < 0)
    {
      _ticksTillAction = -1;
      
      if(OnPressedCallback)
      {
        OnPressedCallback();
      }
      return true;
    }
  }
  
  return false;
}

bool C5Button::Pressed()
{
  if(_ticksTillAction < 0)
  {
    _ticksTillAction = 5;
    
    if(IsToggle)
    {
      State ^= true;
      
      return true;
    }
  }
  
  return IsPressed();
}

bool C5Button::IsPressed()
{
  if(IsToggle)
  {
    return State;
  }
  else
  {
    return (_ticksTillAction >= 0);
  }
}
