#include <ITDB02_Touch.h>
#include "math.h"
#include <ITDB02_Graph16.h>
#include <Wire.h>
#include "RTClib.h"
#include <PWM.h>

#include "SimpleTimer.h"
#include "C5_ScreenOptions.h"
#include "C5_ScreenLock.h"
#include "C5_ScreenDateTime.h"
#include "C5_Button.h"
#include "C5_Visual.h"

#define MEMORY_PIN 0
#define MEMORY_ODO 4

// Declare which fonts we will be using
extern uint8_t BigFont[];
extern uint8_t SmallFont[];
extern uint8_t arialnumbers[];

extern prog_uint16_t backgrad[];
extern prog_uint16_t degreestext[];
extern prog_uint16_t mph[];
extern prog_uint16_t fan0[];
extern prog_uint16_t fan1[];
extern prog_uint16_t fan2[];
extern prog_uint16_t fan3[];
extern prog_uint16_t fan4[];
extern prog_uint16_t fan5[];
extern prog_uint16_t fan6[];
extern prog_uint16_t fan7[];
extern prog_uint16_t fan8[];
extern prog_uint16_t cfive[];
extern prog_uint16_t lighton[];
extern prog_uint16_t lightoff[];
extern prog_uint16_t bluelightoff[];
extern prog_uint16_t bluelighton[];

int BaseUpdateDelay = 50;
int AmpUpdateDelay = 1;

ITDB02 myGLCD(38,39,40,41);
ITDB02_Touch myTouch(6,5,4,3,2);
RTC_DS1307 RTC;

C5ScreenOptions ScreenOptions(&myGLCD, BigFont, SmallFont, arialnumbers);
C5ScreenLock ScreenLock(&myGLCD, BigFont, SmallFont);
C5ScreenDateTime ScreenDateSet(&myGLCD, BigFont, SmallFont, arialnumbers);

int _motorTemp = 0;
int _ambientTemp = 0;
int _mosfetTemp = 0;
int _speed = 0;
int _fanAngle = 0;
int _voltage = 0;
int _amps = 0;
unsigned long _odo = 0;
double _odoSession = 0;

// Pin assignments
int _motorTempPin = 3;
int _ambientTempPin = 0;
int _mosfetTempPin = 4;
int _voltagePin = 1;
int _fanPin = 8;
int _motorMosfet = 9;
int _motorRelayPin = 17;
int _throttlePin = 2;
int _reedPin = 18;
int _handlebarSwitch = 19;
int _peddlePin = 20;
int _underneathLightsPin = 13;
int _mainLightsPin = 12;
int _leftIndicatorPin = 11;
int _rightIndicatorPin = 10;
int _leftIndicatorSwitchPin = 15;
int _rightIndicatorSwitchPin = 16;
int _ampPin = 7;

int _ticksTillNextTempUpdate = 0;
double _ambientTempSum = 0;
int _ticksTillNextMotorTempUpdate = 0;
double _motorTempSum = 0;
int _ticksTillNextMosfetTempUpdate = 0;
double _mosfetTempSum = 0;
int _ticksTillNextVoltageUpdate = 0;
double _voltageSum = 0;
int _ticksTillNextAmpUpdate = 0;
double _ampSum = 0;
const int BaseSpeedUpdateDelay = 4;
int _ticksTillNextSpeedUpdate = 0;
double _speedSum = 0;

int lastBatteryRenderType = -1;
int buttonState = LOW;
int motorState = LOW;
bool isVisible = true;

int lastSec = 9999;
int lastMin = 9999;
int lastHour = 9999;
int lastDay = 9999;
int lastMonth = 9999;
int lastYear = 9999;
boolean timeDrawn = false;

unsigned long ticks = 0;
unsigned long lastReedHitTicks = 0;

C5Button MotorTempButton;
C5Button OutsideTempButton;
C5Button SpeedButton;
C5Button DistButton;
C5Button MainLightsButton;
C5Button HazardLightsButton;
C5Button UnderLightsButton;
C5Button OptionsButton;
C5Button AmpsButton;

bool ignoreNextReedHit = false;
int lastTimeDiff = 0;
bool redrawSpeed = false;
double cmTravelled = 0;
int ticksTillDistRedraw = 0;

SimpleTimer timer;
int leftIndicatorState = LOW;
int rightIndicatorState = LOW;
boolean hazardsOn = false;
boolean flasherState = false;
boolean redrawIndicators = false;
boolean lightsOn = false;
boolean underlightsOn = false;

int lastThrottleNo = -1;

int32_t pwmFrequencyLights = 3000;
int32_t pwmFrequencyMotor = 5000;
int32_t pwmFrequencyCoolingFan = 11000;

boolean isLocked = true;
C5Color mosfetButtonColor(202,134,132);

void setup()
{
  Serial.begin(9600);
  InitTimersSafe();
  
  bool success = SetPinFrequencySafe(_motorMosfet, pwmFrequencyMotor);
  if(success) {
    Serial.println("Set Motor MOSFET Pin Frequency Success"); 
  }
  success = SetPinFrequencySafe(_fanPin, pwmFrequencyCoolingFan);
  if(success) {
    Serial.println("Set Cooling Fan Pin Frequency Success"); 
  }
  
  randomSeed(analogRead(0));

  pinMode(_fanPin, OUTPUT);
  pinMode(_throttlePin, INPUT);
  pinMode(_motorRelayPin, OUTPUT);
  pinMode(_reedPin, INPUT);
  pinMode(_peddlePin, INPUT);
  
  pinMode(_leftIndicatorSwitchPin, INPUT);
  pinMode(_rightIndicatorSwitchPin, INPUT);
  
  pinMode(_mainLightsPin, OUTPUT);
  pinMode(_leftIndicatorPin, OUTPUT);
  pinMode(_rightIndicatorPin, OUTPUT);

  attachInterrupt(4, handlebarSwitch, CHANGE);
  attachInterrupt(5, reedHit, FALLING);

  Wire.begin();
  RTC.begin();
  
  if (!RTC.isrunning()) {
    RTC.adjust(DateTime(__DATE__, __TIME__));
    RAMReset();
  }

  myGLCD.InitLCD(LANDSCAPE);
  myTouch.InitTouch();
  myTouch.setPrecision(PREC_MEDIUM);

  // Attach event handlers
  ScreenLock.OnClosedCallback = ScreenLock_Closed;
  OptionsButton.OnPressedCallback = OptionsButton_Pressed;
  ScreenDateSet.OnClosedCallback = ScreenDateSet_Closed;
  ScreenOptions.OnClosedCallback = ScreenOptions_Closed;
  ScreenOptions.ButtonDate.OnPressedCallback = ScreenOptions_SetDate;
  ScreenOptions.ButtonSetPin.OnPressedCallback = ScreenOptions_SetPin;

  ScreenLock.SetButton(&MotorTempButton, BigFont, 5, 69, 90, 50, "", ScreenLock.BlueButton, ScreenLock.White, ScreenLock.BlueButton, ScreenLock.White);
  ScreenLock.SetButton(&OutsideTempButton, BigFont, 100, 69, 90, 50, "", ScreenLock.BlueButton, ScreenLock.White, ScreenLock.BlueButton, ScreenLock.White);
  OutsideTempButton.IsToggle = true;
  ScreenLock.SetButton(&SpeedButton, BigFont, 195, 124, 120, 50, "", ScreenLock.GreenButton, ScreenLock.White, ScreenLock.GreenButton, ScreenLock.White);
  ScreenLock.SetButton(&DistButton, BigFont, 195, 69, 120, 50, "", ScreenLock.BlueButton, ScreenLock.White, ScreenLock.BlueButton, ScreenLock.White);
  ScreenLock.SetButton(&OptionsButton, BigFont, 5, 124, 90, 50, "", ScreenLock.BlueButton, ScreenLock.White, ScreenLock.BlueButton, ScreenLock.White);
  ScreenLock.SetButton(&AmpsButton, BigFont, 100, 124, 90, 50, "", ScreenLock.BlueButton, ScreenLock.White, ScreenLock.BlueButton, ScreenLock.White);
  
  ScreenLock.SetButton(&MainLightsButton, BigFont, 82, 186, 48, 48, "", C5Color(128,128,128), ScreenLock.White, C5Color(160,160,160), ScreenLock.White);
  MainLightsButton.IsToggle = true;
  ScreenLock.SetButton(&HazardLightsButton, BigFont, 136, 186, 48, 48, "", C5Color(165,87,81), ScreenLock.White, C5Color(202,134,132), ScreenLock.White);
  HazardLightsButton.IsToggle = true;
  ScreenLock.SetButton(&UnderLightsButton, BigFont, 189, 186, 48, 48, "", C5Color(128,128,128), ScreenLock.White, C5Color(160,160,160), ScreenLock.White);
  UnderLightsButton.IsToggle = true;

  FirstTimeInit();
  ShowUnlockScreen();
  timer.setInterval(500, flasherChange);
}

void FirstTimeInit()
{
  _odo = RAMGetODO();
}

void DrawHazardLightsButton()
{
  C5Position2 offset;
  HazardLightsButton.Render(&myGLCD, offset);

  DrawHazardLight();
}

void DrawMainLightsButton()
{
  C5Position2 offset;
  MainLightsButton.Render(&myGLCD, offset);
  
  DrawMainLight();
}

void DrawUnderLightsButton()
{
  C5Position2 offset;
  UnderLightsButton.Render(&myGLCD, offset);
  
  DrawUnderLight();
}

void DrawMotorTempButton()
{
  C5Position2 offset;

  MotorTempButton.Render(&myGLCD, offset);

  myGLCD.drawBitmap(67, 75, 24, 16, (unsigned int*)degreestext);
  DrawMotorTemp();
}

void DrawOutsideTempButton()
{
  C5Position2 offset;
  OutsideTempButton.Render(&myGLCD, offset);

  myGLCD.drawBitmap(167, 75, 24, 16, (unsigned int*)degreestext);
  if(OutsideTempButton.State)
  {
    DrawMosfetTemp();
  }
  else
  {
    DrawAmbientTemp();
  }
}

void DrawSpeedButton()
{
  C5Position2 offset;
  SpeedButton.Render(&myGLCD, offset);
  DrawSpeed();

  myGLCD.drawBitmap(274, 148, 32, 16, (unsigned int*)mph);
}

void DrawDistButton()
{
  C5Position2 offset;
  DistButton.Render(&myGLCD, offset);
  DrawDist();
}

void DrawOptionsButton()
{
  C5Position2 offset;

  OptionsButton.Render(&myGLCD, offset);

  myGLCD.drawBitmap(OptionsButton.Position.X + 13, OptionsButton.Position.Y + 9, 64, 32, (unsigned int*)cfive);
}

void DrawAmpsButton()
{
  C5Position2 offset;

  AmpsButton.Render(&myGLCD, offset);

  DrawAmps();
  
}

void DrawMainScreen()
{
  isVisible = true;
  timeDrawn = false;
  lastBatteryRenderType = -1;
  redrawIndicators = true;
  lastThrottleNo = -1;
  
  lastSec = 9999;
  lastMin = 9999;
  lastHour = 9999;
  lastDay = 9999;
  lastMonth = 9999;
  lastYear = 9999;

  ClearScreen();
  //DrawHUD();

  DrawMotorTempButton();
  DrawOutsideTempButton();
  DrawSpeedButton();
  DrawDistButton();
  DrawBattery();
  //DrawTime();
  DrawDist();
  DrawOptionsButton();
  DrawAmpsButton();
  
  DrawMainLightsButton();
  DrawHazardLightsButton();
  DrawUnderLightsButton();
  
  BaseUpdateDelay = 200;
  _ticksTillNextTempUpdate = BaseUpdateDelay;
  _ticksTillNextMotorTempUpdate = BaseUpdateDelay;
  _ticksTillNextMosfetTempUpdate = BaseUpdateDelay;
  _ticksTillNextVoltageUpdate = BaseUpdateDelay;
  _ambientTempSum = 0;
  _motorTempSum = 0;
  _mosfetTempSum = 0;
  _voltageSum = 0;
}

void loop()
{
  checkTemperatureChanges();
  delay(5);
  checkVoltageChanges();
  delay(5);
  checkAmpChanges();
  delay(5);
  
  if(isVisible)
  {
    checkForFlasherRenderingUpdates();
    DrawFan();
    DrawTime();
    if(redrawSpeed)
    {
      redrawSpeed = false;
      DrawSpeed();
    }
  }
  
  //checkMosfetTemperatureChanges();

  if(!isLocked)
  {
    updateIndicatorButtonState();
    updateFanControl();
    UpdateDistanceTravelled();
  }

  checkInput();

  if(isVisible)
  {
    MotorTempButton.Update();
    OutsideTempButton.Update();
    SpeedButton.Update();
    OptionsButton.Update();
    
    MainLightsButton.Update();
    HazardLightsButton.Update();
    UnderLightsButton.Update();
  }
  else
  {
    ScreenOptions.Update();
    ScreenLock.Update();
    ScreenDateSet.Update();
  }

  checkMotorTemperatureChanges();
  delay(5);
  if(!isLocked)
  {
    //updateMotor();
  }
  
  ticks = millis();
  
  int reedtimediff = (int)(ticks - lastReedHitTicks);
  if(reedtimediff > 5000)
  {
    _ticksTillNextSpeedUpdate = 0;
    _speedSum = 0;
    speedChanged(0);
  }
 
  timer.run();
  delay(5);
}

void checkInput()
{
  if (myTouch.dataAvailable() == true)
  {
    int x = 0;
    int y = 0;
    int count = 0;
    int i;

    for(i=0; i<10; i++)
    {
      if (myTouch.dataAvailable() == true)
      {
        myTouch.read();
        int rx = 320-myTouch.getY();
        int ry = myTouch.getX();

        if(rx > 320 || ry > 240) {
          continue;
        }

        x += rx;
        y += ry;
        count ++;
      }
    }

    if(count > 0)
    {
      x = x / count;
      y = y / count;

      if(isVisible)
      {
        if(ScreenOptions.ButtonTouch(&OptionsButton, x, y))
        {
          DrawOptionsButton();
        }
        if(ScreenOptions.ButtonTouch(&MainLightsButton, x, y))
        {
          lightsOn = MainLightsButton.State;
          if(lightsOn)
          {
            digitalWrite(_mainLightsPin, HIGH);
          }
          else
          {
            digitalWrite(_mainLightsPin, LOW);
          }
          
          DrawMainLightsButton();
        }
        if(ScreenOptions.ButtonTouch(&HazardLightsButton, x, y))
        {
          hazardsOn = HazardLightsButton.State;
          DrawHazardLightsButton();
        }
        if(ScreenOptions.ButtonTouch(&UnderLightsButton, x, y))
        {
          underlightsOn = UnderLightsButton.State;
          if(underlightsOn)
          {
            digitalWrite(_underneathLightsPin, HIGH);
          }
          else
          {
            digitalWrite(_underneathLightsPin, LOW);
          }
          
          DrawUnderLightsButton();
        }
        /*if(ScreenOptions.ButtonTouch(&OutsideTempButton, x, y))
        {
          DrawOutsideTempButton();
        }*/
      }
      else
      {
        ScreenLock.OnTouch(x, y);
        ScreenDateSet.OnTouch(x, y);
        ScreenOptions.OnTouch(x, y);
      }
    }
  }
}

void flasherChange()
{
  flasherState ^= true;
  redrawIndicators = true;
  if(flasherState)
  {
    if(leftIndicatorState == HIGH || hazardsOn)
    {
      digitalWrite(_leftIndicatorPin, HIGH);
    }
    if(rightIndicatorState == HIGH || hazardsOn)
    {
      digitalWrite(_rightIndicatorPin, HIGH);
    }
  }
  else
  {
    digitalWrite(_leftIndicatorPin, LOW);
    digitalWrite(_rightIndicatorPin, LOW);
  }
  
  if(ScreenDateSet.IsVisible)
  {
    ScreenDateSet.DrawDate();
    ScreenDateSet.DrawTime();
  }
}

void updateFanControl()
{
  
  int fanSpeed;
  double temp = _motorTemp - 15;

  if(temp < 0) {
    temp = 0; 
  }
  else if(temp > 20) {
    temp = 20; 
  }

  if(temp > 0) {
    fanSpeed = (int)((double)temp*10) + 55;
    
    analogWrite(_fanPin, fanSpeed);

    _fanAngle+= (int)(temp * 2);
    if(_fanAngle >= 81) {
      _fanAngle = 0; 
    }
  }
  else {
    // Fan motor should be off
    analogWrite(_fanPin, 0);
  }
}

void checkTemperatureChanges() {
  _ticksTillNextTempUpdate--;

  float tempC = analogRead(_ambientTempPin);
  _ambientTempSum += (3 * tempC * 100.0)/1024.0;
  
  if(_ticksTillNextTempUpdate <=0) {
    _ambientTempSum /= BaseUpdateDelay;
    
    int tempD = (int)_ambientTempSum;

    if(_ambientTemp != tempD) {
      _ambientTemp = tempD;
  
      if(!OutsideTempButton.State)
      {
        DrawAmbientTemp();
      }
    }

    _ticksTillNextTempUpdate = BaseUpdateDelay;
  }
}

void checkMotorTemperatureChanges() {
  _ticksTillNextMotorTempUpdate--;

  float tempC = analogRead(_motorTempPin);
  _motorTempSum += (6 * tempC * 100.0)/1024.0;

  if(_ticksTillNextMotorTempUpdate <=0) {
    _motorTempSum /= BaseUpdateDelay;
    int tempD = (int)_motorTempSum;

    if(_motorTemp != tempD) {
      _motorTemp = tempD;
      DrawMotorTemp();
    }

    _ticksTillNextMotorTempUpdate = BaseUpdateDelay;
  }
}

void checkMosfetTemperatureChanges() {
  _ticksTillNextMosfetTempUpdate--;

  float tempC = analogRead(_mosfetTempPin);
  _mosfetTempSum += (6 * tempC * 100.0)/1024.0;

  if(_ticksTillNextMosfetTempUpdate <=0) {
    _mosfetTempSum /= BaseUpdateDelay;
    
    int tempD = (int)_mosfetTempSum;

    if(_mosfetTemp != tempD) {
      _mosfetTemp = tempD;
      if(OutsideTempButton.State)
      {
        DrawMosfetTemp();
      }
    }

    _ticksTillNextMosfetTempUpdate = BaseUpdateDelay;
  }
}

void checkVoltageChanges() {
  int sensorValue = analogRead(_voltagePin);
  _voltageSum += (sensorValue/1023.)*180;
  _ticksTillNextVoltageUpdate--;

  if(_ticksTillNextVoltageUpdate <=0) {
    int v = (int)((_voltageSum /= BaseUpdateDelay) * 10);

    if(_voltage != v) {
      _voltage = v;
      DrawBattery();
    }

    _ticksTillNextVoltageUpdate = BaseUpdateDelay;
    _voltageSum = 0;
  }
}

void checkAmpChanges() {
  double sensorValue = analogRead(_ampPin)-522;
  
  if(sensorValue < 0)
  {
    sensorValue = 0;
  }
  
  _ampSum += sensorValue;
  _ticksTillNextAmpUpdate--;

  if(_ticksTillNextAmpUpdate <=0) {
    _ampSum = (_ampSum /= 20);
    int v = (int)_ampSum;

    if(_amps != v) {
      _amps = v;
      DrawAmps();
    }

    _ticksTillNextAmpUpdate = 20;
    _ampSum = 0;
  }
}

void ClearScreen()
{
  int i;
  int x = 0;
  
  myGLCD.setColor(64, 64, 64);
  myGLCD.fillRect(0, 179, 320, 240);

  for(i=0;i<40; i++, x+= 8)
  {
    myGLCD.blit(x, 0, 8, 179, (unsigned int *)backgrad);
  }
}

void DrawBattery()
{
  if(!isVisible)
  {
    return;
  }

  double v = (double)_voltage / 10;
  int x = 252;
  int y = 8;
  int newType = GetBatteryRenderType(v);
  
  if(newType != lastBatteryRenderType)
  {
    // Outline
    myGLCD.setColor(66, 69, 74);
    myGLCD.drawRect(x, y, x+57, y+25);
    myGLCD.drawRect(x+1, y+1, x+56, y+24);
    myGLCD.fillRect(x+56, y+6, x+56+5, y+6+14);
    
    // Backing
    myGLCD.setColor(0, 0, 0);
    myGLCD.fillRect(x+2, y+2, x+55, y+24);
    
    x+= 2;
    y+= 2;
  
    if(newType >= 2)
    {
      myGLCD.setColor(140, 195, 66);
      myGLCD.fillRect(x, y, x+9, y+22);
      x+= 11;
        
      if(newType >= 2) {
        myGLCD.fillRect(x, y, x+9, y+22);
        x+= 11;
      }
      if(newType >= 3) {
        myGLCD.fillRect(x, y, x+9, y+22);
        x+= 11;
      }
      if(newType >= 4) {
        myGLCD.fillRect(x, y, x+9, y+22);
        x+= 11;
      }
      if(newType >= 5) {
        myGLCD.fillRect(x, y, x+9, y+22);
      }
    }
    else
    {
      if(newType == 1) {
        myGLCD.setColor(231, 28, 33);
        myGLCD.fillRect(x, y, x+9, y+22);
      }
    }
    lastBatteryRenderType = newType;
  }

  myGLCD.setFont(BigFont);
  myGLCD.setColor(16, 16, 16);
  myGLCD.setBackColor(198, 186, 198);

  char strVoltage[27];
  doubleToString(strVoltage, v, 1, true);

  myGLCD.printLargeFont(strVoltage, 250, 38);
}

int GetBatteryRenderType(double v)
{
  if(v >= 11.8)
  {
    if(v >= 12.3) {
      return 5;
    }
    if(v >=12.2) {
      return 4;
    }
    if(v >= 12.0) {
      return 3;
    }
    if(v >= 11.8) {
      return 2;
    }
  }
  else
  {
    if(v >= 11.6) {
      return 1;
    }
  }
  
  return 0;
}

int ClampTemperature(int t)
{
  if(t > 99)
  {
    t = 99;
  }
  else if(t <0)
  {
    t = 0;
  }
  
  return t;
}

void DrawMotorTemp()
{
  if(!isVisible)
  {
    return;
  }

  int y = 77;
  int x = 14;
  myGLCD.setFont(arialnumbers);
  myGLCD.setColor(255, 255, 255);
  myGLCD.setBackColor(37, 97, 165);
  int t = ClampTemperature(_motorTemp);

  if(t < 10) {
    myGLCD.setColor(37, 97, 165);
    myGLCD.fillRect(x, y, x+24, y+32);
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x+24, y);
  }
  else {
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x, y);
  }
}

void DrawAmbientTemp()
{
  if(!isVisible)
  {
    return;
  }

  int y = 77;
  int x = 112;
  myGLCD.setFont(arialnumbers);
  myGLCD.setBackColor(37, 97, 165);
  int t = ClampTemperature(_ambientTemp);

  if(t < 10) {
    myGLCD.setColor(37, 97, 165);
    myGLCD.fillRect(x, y, x+23, y+32);
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x+24, y);
  }
  else {
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x, y);
  }
}

void DrawMosfetTemp()
{
  if(!isVisible)
  {
    return;
  }

  int y = 77;
  int x = 112;
  
  myGLCD.setFont(arialnumbers);
  myGLCD.setBackColor(37, 97, 165);
  
  int t = ClampTemperature(_mosfetTemp);

  if(t < 10) {
    myGLCD.setColor(37, 97, 165);
    myGLCD.fillRect(x, y, x+23, y+32);
    myGLCD.setColor(mosfetButtonColor.R, mosfetButtonColor.G, mosfetButtonColor.B);
    myGLCD.printNumI(t, x+24, y);
  }
  else {
    myGLCD.setColor(mosfetButtonColor.R, mosfetButtonColor.G, mosfetButtonColor.B);
    myGLCD.printNumI(t, x, y);
  }
}

void DrawAmps()
{
  if(!isVisible)
  {
    return;
  }

  int x = AmpsButton.Position.X+12;
  int y = AmpsButton.Position.Y+24;
  
  myGLCD.setFont(BigFont);
  myGLCD.setBackColor(37, 97, 165);
  
  int t = _amps;
  if(t < 0) {
   t = 0; 
  }
  else if(t > 999) {
   t = 999; 
  }

  if(t < 10) {
    myGLCD.setColor(37, 97, 165);
    myGLCD.fillRect(x, y, x+32, y+16);
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x+32, y);
  }
  else if(t < 100) {
    myGLCD.setColor(37, 97, 165);
    myGLCD.fillRect(x, y, x+16, y+16);
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x+16, y);
  }
  else {
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(t, x, y);
  }
  
  myGLCD.setFont(BigFont);
  myGLCD.print("a", x+48, y);
}

void DrawSpeed()
{
  if(!isVisible)
  {
    return;
  }

  int y = 132;
  myGLCD.setFont(arialnumbers);

  if(_speed < 10) {
    myGLCD.setColor(SpeedButton.Background.R, SpeedButton.Background.G, SpeedButton.Background.B);
    myGLCD.fillRect(205, y, 205+23, y+32);
    myGLCD.setBackColor(SpeedButton.Background.R, SpeedButton.Background.G, SpeedButton.Background.B);
    myGLCD.setColor(255, 255, 255);
    myGLCD.printNumI(_speed, 205+24, y);
  }
  else if(_speed < 99) {
    myGLCD.setColor(255, 255, 255);
    myGLCD.setBackColor(SpeedButton.Background.R, SpeedButton.Background.G, SpeedButton.Background.B);
    myGLCD.printNumI(_speed, 205, y);
  }
}

void DrawDist()
{
  if(!isVisible)
  {
    return;
  }

  if(ticksTillDistRedraw > 0)
  {
    ticksTillDistRedraw--;
    if(ticksTillDistRedraw > 0)
    {
      return;
    }
  }

  int y = 76;
  char str[10];

  double km = (double)_odo / 1000;
  if(km < 0.1)
  {
    km = 0;
  }
  
  doubleToString(str, km, 1, false);
  String s = str;
  s += " Miles";

  myGLCD.setFont(SmallFont);
  myGLCD.setColor(230, 230, 230);
  myGLCD.setBackColor(DistButton.Background.R, DistButton.Background.G, DistButton.Background.B);

  int length = s.length();
  int width = length*8;
  char temp[10];
  s.toCharArray(temp, 10);
  myGLCD.print(temp, 310-width, y);

  y += 20;

  if(_odoSession < 0.1)
  {
    s = "0.0";
  }
  else
  {
    char str2[10];
    doubleToString(str2, _odoSession, 1, false);
    s = str2;
  }
  s += "M";

  myGLCD.setFont(BigFont);
  myGLCD.setColor(255, 255, 255);
  myGLCD.setBackColor(DistButton.Background.R, DistButton.Background.G, DistButton.Background.B);

  length = s.length();
  width = (13 * (length-1)) + 9;
  char temp2[10];

  s.toCharArray(temp2, 10);
  myGLCD.printLargeFont(temp2, 307-width, y);
}

void DrawFan()
{
  if(!isVisible)
  {
    return;
  }

  int fan = _fanAngle / 10;
  prog_uint16_t* fanData = fan0;

  switch(fan)
  {
  case 0:
    fanData = fan0;
    break;
  case 1:
    fanData = fan1;
    break;
  case 2:
    fanData = fan2;
    break;
  case 3:
    fanData = fan3;
    break;
  case 4:
    fanData = fan4;
    break;
  case 5:
    fanData = fan5;
    break;
  case 6:
    fanData = fan6;
    break;
  case 7:
    fanData = fan7;
    break;
  case 8:
    fanData = fan8;
    break;
  }

  myGLCD.drawBitmap(65, 91, 24, 24, (unsigned int*)fanData);
}

void DrawThrottle()
{
  return;
  
  int x = 200;
  int y = 7;
  int i = 0;
  int a = 0;
  
  for(i=20; i>=4; i-=2, a++)
  {
    myGLCD.setColor(66, 69, 74);
    myGLCD.fillRect(x, y+i, x+4, y+24);
    
    if(a < lastThrottleNo)
    {
      myGLCD.setColor(88, 240, 88);
      myGLCD.fillRect(x+1, y+i+1, x+3, y+23);
    }
    
    x+= 5;
  }
}

void checkForFlasherRenderingUpdates()
{
  if(redrawIndicators)
  {
    redrawIndicators = false;
    
    if(flasherState)
    {
      myGLCD.setColor(255, 198, 0);
      DrawIndicatorArrows((leftIndicatorState == HIGH || hazardsOn), (rightIndicatorState == HIGH || hazardsOn));
    }
    else
    {
      myGLCD.setColor(90, 90, 90);
      DrawIndicatorArrows(true, true);
    }
  }
}

void DrawIndicatorArrows(bool drawLeft, bool drawRight)
{
  int leftIndicator[] = {21,2, 20,3, 19,4, 18,5, 17,6, 16,7, 15,8, 14,9, 13,10, 12,11, 11,12, 10,13, 9,14, 8,15, 7,53, 6,54, 5,55, 4,56, 3,57, 2,58, 1,59, 0,60};
  int i;
  int x;
  int y1;
  int y2;

  if(drawLeft)
  {
    x = 8;
    y1 = 232-43;
    y2 = 232;
    
    for(i=0; i<44; i+=2)
    {
      myGLCD.drawLine(x + leftIndicator[i], y1, x + leftIndicator[i] + leftIndicator[i+1], y1);
      myGLCD.drawLine(x + leftIndicator[i], y2, x + leftIndicator[i] + leftIndicator[i+1], y2);
      
      y1++;
      y2--;
    }
  }
  if(drawRight)
  {
    x = 320-68;
    y1 = 232-43;
    y2 = 232;
    
    for(i=0; i<44; i+=2)
    {
      myGLCD.drawLine(x + (60-leftIndicator[i]), y1, x + (60-(leftIndicator[i] + leftIndicator[i+1])), y1);
      myGLCD.drawLine(x + (60-leftIndicator[i]), y2, x + (60-(leftIndicator[i] + leftIndicator[i+1])), y2);
      
      y1++;
      y2--;
    }
  }
}

void DrawMainLight()
{
  if(!isVisible)
  {
    return;
  }
  
  if(lightsOn)
  {
    myGLCD.drawBitmap(MainLightsButton.Position.X + 12, MainLightsButton.Position.Y + 8, 24, 32, (unsigned int*)lighton);
  }
  else
  {
    myGLCD.drawBitmap(MainLightsButton.Position.X + 12, MainLightsButton.Position.Y + 8, 24, 32, (unsigned int*)lightoff);
  }
}

void DrawHazardLight()
{
  if(!isVisible)
  {
    return;
  }
  
  int x = HazardLightsButton.Position.X+8;
  int y = HazardLightsButton.Position.Y+8;
  int yBot = y+33;
  
  if(hazardsOn)
  {
    myGLCD.setColor(255, 255, 255);
  }
  else
  {
    myGLCD.setColor(160, 160, 160);
  }
  
  myGLCD.drawLine(x+16, y-1, x-1, yBot);
  myGLCD.drawLine(x-1, yBot, x+33, yBot);
  myGLCD.drawLine(x+33, yBot, x+16, y-1);
  
  y += 5;
  yBot -= 3;
  
  myGLCD.drawLine(x+16, y, x+4, yBot);
  myGLCD.drawLine(x+4, yBot, x+28, yBot);
  myGLCD.drawLine(x+28, yBot, x+16, y);
}

void DrawUnderLight()
{
  if(!isVisible)
  {
    return;
  }
  
  if(underlightsOn)
  {
    myGLCD.drawBitmap(UnderLightsButton.Position.X + 8, UnderLightsButton.Position.Y + 16, 32, 16, (unsigned int*)bluelighton);
  }
  else
  {
    myGLCD.drawBitmap(UnderLightsButton.Position.X + 8, UnderLightsButton.Position.Y + 16, 32, 16, (unsigned int*)bluelightoff);
  }
}

void DrawTime()
{
  if(!isVisible)
  {
    return;
  }

  myGLCD.setFont(BigFont);
  myGLCD.setColor(16, 16, 16);
  myGLCD.setBackColor(189, 195, 189);

  DateTime now = RTC.now();
  int thisSec = now.second();
  int thisMin = now.minute();
  int thisHour = now.hour();
  int thisDay = now.day();
  int thisMonth = now.month();
  int thisYear = now.year();
  int charWidth = 16;
  int x = 10;
  int y = 14;
  char str[7];

  if(!timeDrawn)
  {
    myGLCD.print(":", x + ((charWidth * 2)-5), y);
    myGLCD.print(":", x + ((charWidth * 4)+1), y);
    timeDrawn = true;
  }

  if(lastHour != thisHour)
  {
    if(thisHour < 10)
    {
      str[0] = '0';
      itoa(thisHour, str+1, 10);
    }
    else
    {
      itoa(thisHour, str, 10);
    }

    myGLCD.print(str, x, y);
    lastHour = thisHour;
  }

  x+= charWidth * 2;
  x+= 6;

  if(lastMin != thisMin)
  {
    if(thisMin < 10)
    {
      str[0] = '0';
      itoa(thisMin, str+1, 10);
    }
    else
    {
      itoa(thisMin, str, 10);
    }

    myGLCD.print(str, x, y);
    lastMin = thisMin;
  }

  x+= charWidth * 2;
  x+= 6;

  if(lastSec != thisSec)
  {
    if(thisSec < 10)
    {
      str[0] = '0';
      itoa(thisSec, str+1, 10);
    }
    else
    {
      itoa(thisSec, str, 10);
    }

    myGLCD.print(str, x, y);
    lastSec = thisSec;
  }

  if(thisDay != lastDay || thisMonth != lastMonth || thisYear != lastYear)
  {
    String dayString = "";
    char temp[16];

    itoa(thisDay, temp, 10);
    dayString += temp;

    switch(thisDay)
    {
    case 1:
    case 21:
    case 31:
      dayString += "ST ";
      break;
    case 2:
    case 22:
      dayString += "ND ";
      break;
    case 3:
    case 23:
      dayString += "RD ";
      break;
    default:
      dayString += "TH ";
      break;
    }

    switch(thisMonth)
    {
    case 1:
      dayString += "JANUARY ";
      break;
    case 2:
      dayString += "FEBRUARY ";
      break;
    case 3:
      dayString += "MARCH ";
      break;
    case 4:
      dayString += "APRIL ";
      break;
    case 5:
      dayString += "MAY ";
      break;
    case 6:
      dayString += "JUNE ";
      break;
    case 7:
      dayString += "JULY ";
      break;
    case 8:
      dayString += "AUGUST ";
      break;
    case 9:
      dayString += "SEPTEMBER ";
      break;
    case 10:
      dayString += "OCTOBER ";
      break;
    case 11:
      dayString += "NOVEMBER ";
      break;
    case 12:
      dayString += "DECEMBER ";
      break;
    }

    //Serial.println(dayString);
    //Serial.println(thisYear);

    dayString += thisYear;

    //Serial.println(dayString);

    char date[100];
    dayString.toCharArray(date, 100);

    //Serial.println(date);

    myGLCD.setBackColor(198, 190, 198);
    myGLCD.setFont(SmallFont);

    myGLCD.print(date, 10, 36);



    lastDay = thisDay;
    lastMonth = thisMonth;
    lastYear = thisYear;
  }
}

void doubleToString(char *st, double num, byte dec, boolean appendV)
{
  char buf[25];
  boolean neg=false;
  int c=0;
  int c2;
  unsigned long inum;

  if (num==0)
  {
    st[0]=48;
    st[1]=46;
    for (int i=0; i<dec; i++)
      st[2+i]=48;
    if(appendV) {
      st[2+dec]='V';
      st[3+dec]=0;
    }
    else {
     st[2+dec]=0; 
    }
  }
  else
  {
    if (num<0)
    {
      neg=true;
      num=-num;
    }

    if (dec<1)
      dec=1;
    if (dec>5)
      dec=5;

    inum=long(num*pow(10,dec));

    while (inum>0)
    {
      buf[c]=48+(inum % 10);
      c++;
      inum=(inum-(inum % 10))/10;
    }
    if ((num<1) and (num>0))
    {
      buf[c]=48;
      c++;
    }
    buf[c]=0;

    if (neg)
    {
      st[0]=45;
    }

    c2=neg;
    for (int i=0; i<c; i++)
    {
      st[c2]=buf[c-i-1];
      c2++;
      if ((c-(c2-neg))==dec)
      {
        st[c2]=46;
        c2++;
      }
    }
    if(appendV) {
      st[c2]='V';
      c2++;
    }
    st[c2]=0;
  }
}

String PadString(String s, int num, char c)
{
  int zeros = num - s.length();
  int i;
  String result = "";

  for(i=0; i<zeros; i++)
  {
    result += c;
  }

  result += s;
  return result;
}

void ShowPinSetScreen()
{
  ScreenLock.PinEntered = "";
  ScreenLock.SetMode = true;
  ShowScreen(&ScreenLock);
}

void ShowUnlockScreen()
{
  isLocked = true;
  ScreenLock.Pin = "";
  ScreenLock.PinEntered = "";
  ScreenLock.SetMode = false;
  ShowScreen(&ScreenLock);
}

void ShowDateTimeSetScreen()
{
  ShowScreen(&ScreenDateSet);
  ScreenDateSet.Init(&RTC);
}

void ShowOptionsScreen()
{
  ShowScreen(&ScreenOptions);
}

void ShowScreen(C5Screen *pScreen)
{
  isVisible = false;
  pScreen->IsVisible = true;
  pScreen->IsEnabled = true;
  pScreen->Render();
}

void UpdateDistanceTravelled()
{
  if(cmTravelled > 200)
  {
    int m = (int)(cmTravelled/100);
    cmTravelled -= (m*100);
    unsigned long newODO = RAMGetODO() + m;

    if(_odo != newODO)
    {
      DrawDist();
    }

    _odo = newODO;
    RAMSetODO(_odo);
  }
}

// Persisent RAM
void RAMReset()
{
  RAMSetPin("1234");
  RAMSetODO(17L);
}

void RAMSetPin(char *pin)
{
  WriteRam(MEMORY_PIN, pin[0]);
  WriteRam(MEMORY_PIN+1, pin[1]);
  WriteRam(MEMORY_PIN+2, pin[2]);
  WriteRam(MEMORY_PIN+3, pin[3]);
}

void RAMSetODO(unsigned int m)
{
  WriteRam(MEMORY_ODO, (byte)(m & 255));
  WriteRam(MEMORY_ODO+1, (byte)((m>>8) & 255));
  WriteRam(MEMORY_ODO+2, (byte)((m>>16) & 255));
  WriteRam(MEMORY_ODO+3, (byte)((m>>24) & 255));
}

unsigned long RAMGetODO()
{
  unsigned int m = 0;

  m = ReadRam(MEMORY_ODO);
  m |= ReadRam(MEMORY_ODO+1) << 8;
  m |= ReadRam(MEMORY_ODO+2) << 16;
  m |= ReadRam(MEMORY_ODO+3) << 24;

  return m;
}

void WriteRam(int address, byte data){
  Wire.beginTransmission(0x68);              // Select DS1307
  Wire.write(address+8);                       // address location starts at 8, 0-6 are date, 7 is control
  Wire.write(data);                            // send data
  Wire.endTransmission();
}

byte ReadRam(int address){
  Wire.beginTransmission(0x68);
  Wire.write(address+8);
  Wire.endTransmission();

  Wire.requestFrom(0x68, 1);
  return Wire.read();
}

// Event Handlers
void ScreenDateSet_Closed()
{
  Serial.println("ScreenDateSet_Closed");
  ShowOptionsScreen();
}

void ScreenOptions_Closed()
{
  Serial.println("");
  DrawMainScreen();
}

void ScreenOptions_SetDate()
{
  ShowDateTimeSetScreen();
}

void ScreenOptions_SetPin()
{
  ShowPinSetScreen();
}

void ScreenLock_Closed()
{
  char temp[5];
  ScreenLock.Pin.toCharArray(temp, 5);

  /*Serial.print("Pin Entered:");
   Serial.print(temp[0]);
   Serial.print(temp[1]);
   Serial.print(temp[2]);
   Serial.println(temp[3]);*/

  if(ScreenLock.SetMode)
  {
    if(ScreenLock.Pin.length() == 4)
    {
      RAMSetPin(temp);
    }
    ShowOptionsScreen();
  }
  else
  {
    /*Serial.print("Pin In RAM:");
     Serial.print((char)ReadRam(MEMORY_PIN));
     Serial.print((char)ReadRam(MEMORY_PIN+1));
     Serial.print((char)ReadRam(MEMORY_PIN+2));
     Serial.println((char)ReadRam(MEMORY_PIN+3));*/

    if((char)ReadRam(MEMORY_PIN) == temp[0] && (char)ReadRam(MEMORY_PIN+1) == temp[1] && (char)ReadRam(MEMORY_PIN+2) == temp[2] && (char)ReadRam(MEMORY_PIN+3) == temp[3])
    {
      DrawMainScreen();
      isLocked = false;
    }
    else
    {
      ShowUnlockScreen();
    }
  }
}

void OptionsButton_Pressed()
{
  ShowOptionsScreen();
}

// Interrupts
void reedHit()
{
  detachInterrupt(5);
  int timediff = (int)(ticks - lastReedHitTicks);

  //Serial.print("Hit: ");
  //Serial.println(timediff);
  
  double v = (double)timediff / (double)lastTimeDiff;
  //Serial.print("Change: ");
  //Serial.println(v);
  
  if(v > 0.5)
  {   // Were only interested in changes over the longer period
    double d = 0.0008;
    double hours = (((double)timediff / 1000) / 60) / 60;
    double f = ((d / hours) * 2) * 0.621371192;
    
    _speedSum+= f;
    _ticksTillNextSpeedUpdate--;
    
    if(_ticksTillNextSpeedUpdate <=0) {
      _speedSum /= BaseSpeedUpdateDelay;
      speedChanged((int)f);
  
      _ticksTillNextSpeedUpdate = BaseSpeedUpdateDelay;
    }
    
    cmTravelled += (d*100000);
    _odoSession += d;
    
    Serial.print("_odoSession: ");
    Serial.println(_odoSession);
  }
  
  lastReedHitTicks = ticks;
  lastTimeDiff = timediff;
  
  attachInterrupt(5, reedHit, FALLING);
}

void speedChanged(int newSpeed)
{
  if(newSpeed != _speed)
  {
    _speed = newSpeed;
    redrawSpeed = true;
    
    //Serial.print("Speed: ");
    //Serial.println(_speed);
  }
}
/*
void updateMotor()
{
  double val=0;
  int i;
  
  // Take 5 samples from the throttle input and take the average
  for(i=0; i<5; i++)
  {
    val+= analogRead(_throttlePin);
    delay(1);
  }
  
  val /= 5;
  
  val -= 170;
  val *= 0.0014164305949008;
  
  if(val <= 0)
  {
    val = 0;
  }
  else if(val > 1)
  {
    val = 1;
  }
  
  int throttle = (val * 200);
  if(throttle < 5)
  {
    throttle = 0;
  }
  else if(throttle > 195)
  {
    throttle = 200;
  }
  
  int newThrottleVal = 0;
  
  if(throttle == 0)
  {
    pwmWrite(_motorMosfet, 0);
  }
  else
  {
    pwmWrite(_motorMosfet, 50 + throttle);
    newThrottleVal = throttle / 12; 
  }
  
  if(newThrottleVal != lastThrottleNo)
  {
    lastThrottleNo = newThrottleVal;
    DrawThrottle();
  }
}*/

void handlebarSwitch() {
  if(!isLocked)
  {
    buttonState = digitalRead(_handlebarSwitch);
  
    if(buttonState == HIGH) {
      if(motorState == LOW) {
        digitalWrite(_motorRelayPin, HIGH);
        motorState = HIGH;
      } 
    }
    else {
      if(motorState == HIGH) {
        digitalWrite(_motorRelayPin, LOW);
        motorState = LOW;
      } 
    }
  }
  else
  {
    digitalWrite(_motorRelayPin, LOW);
  }
}

void updateIndicatorButtonState()
{
  leftIndicatorState = digitalRead(_leftIndicatorSwitchPin);
  rightIndicatorState = digitalRead(_rightIndicatorSwitchPin);
}
