#include <ITDB02_Touch.h>
#include <ITDB02_Graph16.h>

#include "C5_Visual.h"
#include "C5_Screen.h"

C5Screen::C5Screen()
{
  Reset();
}

void C5Screen::Reset()
{
  C5Visual::Reset();
  
  BlueButton = C5Color(37, 97, 165);
  BluePressedButton = C5Color(77, 137, 205);
  GreenButton = C5Color(35, 189, 35);
  White = C5Color(255,255,255);
  Black = C5Color(0,0,0);
  
  Position.X = 0;
  Position.Y = 0;
  Size.Width = 320;
  Size.Height = 240;
  IsVisible = false;
  
  _ticksTillClose = 0;
  
  TextColor = C5Color(255,255,255);
}

void C5Screen::Render()
{
  pLCD->setColor(Background.R, Background.G, Background.B);
  pLCD->fillRect(0, 0, Size.Width, Size.Height);
}

bool C5Screen::Update()
{
  if(IsVisible)
  {
    if(_ticksTillClose > 0)
    {
      _ticksTillClose--;
      if(_ticksTillClose == 0)
      {
        Close();
      }
    }
  }
  
  return false;
}

void C5Screen::OnTouch(int x, int y)
{
}

boolean C5Screen::ButtonTouch(C5Button *pButton, int x, int y)
{
  if(pButton->HitTest(x, y, 0))
    {
      if(pButton->Pressed())
      {
        if(pButton->RedrawOnPress)
        {
          pButton->Render(pLCD, Position);
        }
        return true;
      }
    }
    
    return false;
}

void C5Screen::SetButton(C5Button *pButton, uint8_t *pFont, int x, int y, int width, int height, char *text, C5Color background, C5Color color, C5Color pressedBackground, C5Color pressedColor)
{
  pButton->pFont = pFont;
  pButton->Background = background;
  pButton->Color = color;
  pButton->Position.X = x;
  pButton->Position.Y = y;
  pButton->Size.Width = width;
  pButton->Size.Height = height;
  pButton->Text = text;
  
  pButton->BackgroundSelected = pressedBackground;
  pButton->ColorSelected = pressedColor;
  pButton->TextLength = strlen(pButton->Text);
}

void C5Screen::Close()
{
  IsVisible = false;
  if(OnClosedCallback)
  {
    OnClosedCallback();
  }
}

