#include <ITDB02_Touch.h>
#include <ITDB02_Graph16.h>

#include "C5_Visual.h"
#include "C5_ScreenDateTime.h"
#include <Wire.h>
#include "RTClib.h"

C5ScreenDateTime::C5ScreenDateTime(ITDB02 *plcd, uint8_t *bigFont, uint8_t *smallFont, uint8_t *sevenSegNumFont)
{
  pLCD = plcd;
  pBigFont = bigFont;
  pSmallFont = smallFont;
  pSevenSegNumFont = sevenSegNumFont;
  
  Reset();
}

void C5ScreenDateTime::Reset()
{
  C5Screen::Reset();
  
  Background = C5Color(0,0,0);
  C5Color ButtonTextColor(200,200,200);
  C5Color PressedButtonTextColor(255,255,255);
  
  SetButton(&DecYear, pBigFont, 16, 80, 32, 32, "-", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&IncYear, pBigFont, 16, 40, 32, 32, "+", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&DecMonth, pBigFont, 80, 80, 32, 32, "-", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&IncMonth, pBigFont, 80, 40, 32, 32, "+", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&DecDay, pBigFont, 128, 80, 32, 32, "-", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&IncDay, pBigFont, 128, 40, 32, 32, "+", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  
  SetButton(&DecHour, pBigFont, 192, 80, 32, 32, "-", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&IncHour, pBigFont, 192, 40, 32, 32, "+", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&DecMinute, pBigFont, 240, 80, 32, 32, "-", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&IncMinute, pBigFont, 240, 40, 32, 32, "+", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  
  SetButton(&ButtonClose, pBigFont, 196, 240 - 60, 108, 48, "Exit", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
}

void C5ScreenDateTime::Render()
{
  C5Screen::Render();
  
  DecYear.Render(pLCD, Position);
  IncYear.Render(pLCD, Position);
  DecMonth.Render(pLCD, Position);
  IncMonth.Render(pLCD, Position);
  DecDay.Render(pLCD, Position);
  IncDay.Render(pLCD, Position);
  
  DecHour.Render(pLCD, Position);
  IncHour.Render(pLCD, Position);
  DecMinute.Render(pLCD, Position);
  IncMinute.Render(pLCD, Position);
  
  ButtonClose.Render(pLCD, Position);
  
  //pLCD->setColor(37, 97, 165);
  //pLCD->fillRoundRect(10, 88, 170, 88+64);
}

bool C5ScreenDateTime::Update()
{
  DecYear.Update();
  IncYear.Update();
  DecMonth.Update();
  IncMonth.Update();
  DecDay.Update();
  IncDay.Update();
  DecHour.Update();
  IncHour.Update();
  DecMinute.Update();
  IncMinute.Update();
  
  if(ButtonClose.Update())
  {
    Close();
  }
  
  return false;
}

void C5ScreenDateTime::OnTouch(int x, int y)
{
  if(IsVisible)
  {
    DateTime now = pRTC->now();
    
    if(DecYear.HitTest(x, y, 0))
    {
      if(DecYear.Pressed())
      {
        DateChanged(&DecYear, thisYear-1, now.month(), now.day(), now.hour(), now.minute(), now.second());
      }
    }
    if(IncYear.HitTest(x, y, 0))
    {
      if(IncYear.Pressed())
      {
        DateChanged(&IncYear, thisYear+1, now.month(), now.day(), now.hour(), now.minute(), now.second());
      }
    }
    if(DecMonth.HitTest(x, y, 0))
    {
      if(DecMonth.Pressed())
      {
        DateChanged(&DecMonth, now.year(), thisMonth-1, now.day(), now.hour(), now.minute(), now.second());
      }
    }
    if(IncMonth.HitTest(x, y, 0))
    {
      if(IncMonth.Pressed())
      {
        DateChanged(&IncMonth, now.year(), thisMonth+1, now.day(), now.hour(), now.minute(), now.second());
      }
    }
    if(DecDay.HitTest(x, y, 0))
    {
      if(DecDay.Pressed())
      {
        DateChanged(&DecDay, now.year(), now.month(), thisDay-1, now.hour(), now.minute(), now.second());
      }
    }
    if(IncDay.HitTest(x, y, 0))
    {
      if(IncDay.Pressed())
      {
        DateChanged(&IncDay, now.year(), now.month(), thisDay+1, now.hour(), now.minute(), now.second());
      }
    }
    if(DecHour.HitTest(x, y, 0))
    {
      if(DecHour.Pressed())
      {
        DateChanged(&DecHour, now.year(), now.month(), now.day(), thisHour-1, now.minute(), now.second());
      }
    }
    if(IncHour.HitTest(x, y, 0))
    {
      if(IncHour.Pressed())
      {
        DateChanged(&IncHour, now.year(), now.month(), now.day(), thisHour+1, now.minute(), now.second());
      }
    }
    if(DecMinute.HitTest(x, y, 0))
    {
      if(DecMinute.Pressed())
      {
        DateChanged(&DecMinute, now.year(), now.month(), now.day(), now.hour(), thisMin-1, now.second());
      }
    }
    if(IncMinute.HitTest(x, y, 0))
    {
      if(IncMinute.Pressed())
      {
        DateChanged(&IncMinute, now.year(), now.month(), now.day(), now.hour(), thisMin+1, now.second());
      }
    }
    
    if(ButtonTouch(&ButtonClose, x, y))
    {
    }
  }
}

void C5ScreenDateTime::DateChanged(C5Button *pButton, int newYear, int newMonth, int newDay, int newHour, int newMinute, int newSecond)
{
  if(newMinute < 0) {
    newMinute = 59;
    newHour--; 
  }
  if(newMinute > 59) {
    newMinute = 0;
    newHour++; 
  }
  if(newHour < 0) {
    newHour = 23;
    newDay--; 
  }
  if(newHour > 23) {
    newHour = 0;
    newDay++; 
  }
  
  if(newMonth < 1) {
    newMonth = 12;
    newYear--; 
  }
  if(newMonth > 12) {
    newMonth = 1;
    newYear++; 
  }
  
  int monthDays[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
  if(newYear % 4 == 0)
  {
    monthDays[1] = 29;
  }
  
  if(newDay < 1) {
    newDay = monthDays[newMonth-1];
    newMonth--; 
  }
  if(newDay > monthDays[newMonth-1]) {
    newDay = 1;
    newMonth++; 
  }
  if(newMonth < 1) {
    newMonth = 12;
    newYear--; 
  }
  if(newMonth > 12) {
    newMonth = 1;
    newYear++; 
  }
  
  pButton->Render(pLCD, Position);
  pRTC->adjust(DateTime(newYear, newMonth, newDay, newHour, newMinute, newSecond));
  DrawDate();
  DrawTime();
}

void C5ScreenDateTime::Init(RTC_DS1307 *prtc)
{
  pRTC = prtc;
  DrawDate();
  DrawTime();
}

void C5ScreenDateTime::DrawDate()
{
  DateTime now = pRTC->now();
  thisDay = now.day();
  thisMonth = now.month();
  thisYear = now.year();
  
  pLCD->setFont(pBigFont);
  pLCD->setColor(255, 255, 255);
  pLCD->setBackColor(0, 0, 0);
  
  int y = 10;
  int x = 0;
  pLCD->printNumI(thisYear, x+0, y);
  pLCD->print("/", x+64, y);
  pLCD->printNumIPad2(thisMonth, x+80, y);
  pLCD->print("/", x+112, y);
  pLCD->printNumIPad2(thisDay, x+128, y);
}

void C5ScreenDateTime::DrawTime()
{
  DateTime now = pRTC->now();
  thisSec = now.second();
  thisMin = now.minute();
  thisHour = now.hour();
  
  pLCD->setFont(pBigFont);
  pLCD->setColor(255, 255, 255);
  pLCD->setBackColor(0, 0, 0);
  
  int y = 10;
  int x = 0;
  
  pLCD->printNumIPad2(thisHour, x+192, y);
  pLCD->print(":", x+224, y);
  pLCD->printNumIPad2(thisMin, x+240, y);
  pLCD->print(":", x+272, y);
  pLCD->printNumIPad2(thisSec, x+288, y);
}
