#include <ITDB02_Touch.h>
#include <ITDB02_Graph16.h>

#include "C5_Visual.h"
#include "C5_ScreenLock.h"

C5ScreenLock::C5ScreenLock(ITDB02 *plcd, uint8_t *bigFont, uint8_t *smallFont)
{
  pLCD = plcd;
  pBigFont = bigFont;
  pSmallFont = smallFont;
  
  Reset();
}

void C5ScreenLock::Reset()
{
  C5Screen::Reset();
  
  Background = Black;
  PinEntered = "";
  
  int y = 60;
  
  SetButton(&Button0, pBigFont, 16, y, 48, 48, "0", BlueButton, White, BluePressedButton, White);
  SetButton(&Button1, pBigFont, 76, y, 48, 48, "1", BlueButton, White, BluePressedButton, White);
  SetButton(&Button2, pBigFont, 136, y, 48, 48, "2", BlueButton, White, BluePressedButton, White);
  SetButton(&Button3, pBigFont, 196, y, 48, 48, "3", BlueButton, White, BluePressedButton, White);
  SetButton(&Button4, pBigFont, 256, y, 48, 48, "4", BlueButton, White, BluePressedButton, White);
  y+= 60;
  
  SetButton(&Button5, pBigFont, 16, y, 48, 48, "5", BlueButton, White, BluePressedButton, White);
  SetButton(&Button6, pBigFont, 76, y, 48, 48, "6", BlueButton, White, BluePressedButton, White);
  SetButton(&Button7, pBigFont, 136, y, 48, 48, "7", BlueButton, White, BluePressedButton, White);
  SetButton(&Button8, pBigFont, 196, y, 48, 48, "8", BlueButton, White, BluePressedButton, White);
  SetButton(&Button9, pBigFont, 256, y, 48, 48, "9", BlueButton, White, BluePressedButton, White);
  
  y = 240 - 60;
  SetButton(&ButtonCancel, pBigFont, 196, y, 108, 48, "Cancel", BlueButton, White, BluePressedButton, White);
}

void C5ScreenLock::Render()
{
  if(IsVisible)
  {
    C5Screen::Render();
    
    pLCD->setFont(pBigFont);
    pLCD->setColor(TextColor.R, TextColor.G, TextColor.B);
    pLCD->setBackColor(Background.R, Background.G, Background.B);
    
    if(SetMode)
    {
      pLCD->print("SET PIN", CENTER, 10);
      pLCD->setFont(pSmallFont);
      pLCD->print("Enter a 4-digit PIN to secure your C5", CENTER, 30);
      ButtonCancel.Render(pLCD, Position);
    }
    else
    {
      pLCD->print("ENTER PIN", CENTER, 10);
      pLCD->print("TO UNLOCK", CENTER, 30);
    }
    
    Button0.Render(pLCD, Position);
    Button1.Render(pLCD, Position);
    Button2.Render(pLCD, Position);
    Button3.Render(pLCD, Position);
    Button4.Render(pLCD, Position);
    Button5.Render(pLCD, Position);
    Button6.Render(pLCD, Position);
    Button7.Render(pLCD, Position);
    Button8.Render(pLCD, Position);
    Button9.Render(pLCD, Position);
    
    RenderPinEntered();
  }
}

bool C5ScreenLock::Update()
{
  if(IsVisible)
  {
    C5Screen::Update();
    
    CheckButtonUpdate(&Button0);
    CheckButtonUpdate(&Button1);
    CheckButtonUpdate(&Button2);
    CheckButtonUpdate(&Button3);
    CheckButtonUpdate(&Button4);
    CheckButtonUpdate(&Button5);
    CheckButtonUpdate(&Button6);
    CheckButtonUpdate(&Button7);
    CheckButtonUpdate(&Button8);
    CheckButtonUpdate(&Button9);
    
    if(SetMode)
    {
      if(ButtonCancel.Update())
      {
        Close();
      }
    }
  }
  
  return false;
}

void C5ScreenLock::OnTouch(int x, int y)
{
  if(IsVisible && IsEnabled)
  {
    CheckButtonTouch(&Button0, x, y);
    CheckButtonTouch(&Button1, x, y);
    CheckButtonTouch(&Button2, x, y);
    CheckButtonTouch(&Button3, x, y);
    CheckButtonTouch(&Button4, x, y);
    CheckButtonTouch(&Button5, x, y);
    CheckButtonTouch(&Button6, x, y);
    CheckButtonTouch(&Button7, x, y);
    CheckButtonTouch(&Button8, x, y);
    CheckButtonTouch(&Button9, x, y);
    if(SetMode)
    {
      ButtonTouch(&ButtonCancel, x, y);
    }
  }
}

void C5ScreenLock::CheckButtonTouch(C5Button *pButton, int x, int y)
{
  if(ButtonTouch(pButton, x, y))
  {
    PinEntered += "*";
    Pin += pButton->Text;
    RenderPinEntered();
    
    if(PinEntered.length() == 4)
    {
      IsEnabled = false;
      _ticksTillClose = 10;
    }
  }
}

void C5ScreenLock::CheckButtonUpdate(C5Button *pButton)
{
  if(pButton->Update())
  {
    pButton->Render(pLCD, Position);
  }
}

void C5ScreenLock::RenderPinEntered()
{
  pLCD->setColor(TextColor.R, TextColor.G, TextColor.B);
  pLCD->setBackColor(Background.R, Background.G, Background.B);
  char temp[100];
  PinEntered.toCharArray(temp, 100);
  
  Serial.println(temp);
  pLCD->print(temp, 16, 240 - 44);
}

