#include <ITDB02_Touch.h>
#include <ITDB02_Graph16.h>

#include "C5_Visual.h"
#include "C5_ScreenOptions.h"

C5ScreenOptions::C5ScreenOptions(ITDB02 *plcd, uint8_t *bigFont, uint8_t *smallFont, uint8_t *sevenSegNumFont)
{
  pLCD = plcd;
  pBigFont = bigFont;
  pSmallFont = smallFont;
  pSevenSegNumFont = sevenSegNumFont;
  
  Reset();
}

void C5ScreenOptions::Reset()
{
  C5Screen::Reset();
  
  C5Color ButtonTextColor(200,200,200);
  C5Color PressedButtonTextColor(255,255,255);
  
  SetButton(&ButtonDate, pBigFont, 50, 20, 220, 40, "Set Date/Time", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&ButtonSetPin, pBigFont, 50, 70, 220, 40, "Set PIN", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&ButtonDisablePin, pBigFont, 50, 120, 220, 40, "Disable PIN", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
  SetButton(&ButtonClose, pBigFont, 12, 240 - 60, 108, 48, "Exit", BlueButton, ButtonTextColor, BluePressedButton, PressedButtonTextColor);
}

void C5ScreenOptions::Render()
{
  C5Screen::Render();
  ButtonDate.Render(pLCD, Position);
  ButtonSetPin.Render(pLCD, Position);
  ButtonDisablePin.Render(pLCD, Position);
  ButtonClose.Render(pLCD, Position);
  
  //pLCD->setColor(37, 97, 165);
  //pLCD->fillRoundRect(10, 88, 170, 88+64);
}

bool C5ScreenOptions::Update()
{
  ButtonDate.Update();
  ButtonSetPin.Update();
  ButtonDisablePin.Update();
  
  if(ButtonClose.Update())
  {
    Close();
  }
  
  return false;
}

void C5ScreenOptions::OnTouch(int x, int y)
{
  if(IsVisible)
  {
    if(ButtonDate.HitTest(x, y, 0))
    {
      if(ButtonDate.Pressed())
      {
        ButtonDate.Render(pLCD, Position);
      }
    }
    if(ButtonSetPin.HitTest(x, y, 0))
    {
      if(ButtonSetPin.Pressed())
      {
        ButtonSetPin.Render(pLCD, Position);
      }
    }
    if(ButtonDisablePin.HitTest(x, y, 0))
    {
      if(ButtonDisablePin.Pressed())
      {
        ButtonDisablePin.Render(pLCD, Position);
      }
    }
    if(ButtonClose.HitTest(x, y, 0))
    {
      if(ButtonClose.Pressed())
      {
        ButtonClose.Render(pLCD, Position);
      }
    }
  }
}
